@@ -159,7 +159,7 @@ func depRead(fdDep *os.File) (*DepInfo, error) {
 	}
 	depInfo := DepInfo{}
 	if b := m["Build"]; b == "" {
-		return nil, errors.New(".dep missing Build:")
+		return nil, errors.New(".rec missing Build:")
 	} else {
 		depInfo.build = b
 	}
@@ -177,7 +177,7 @@ func depRead(fdDep *os.File) (*DepInfo, error) {
 		case DepTypeIfcreate:
 			dep := m["Target"]
 			if dep == "" {
-				return nil, errors.New("invalid format of .dep")
+				return nil, errors.New("invalid format of .rec")
 			}
 			depInfo.ifcreates = append(depInfo.ifcreates, dep)
 		case DepTypeIfchange:
@@ -186,11 +186,11 @@ func depRead(fdDep *os.File) (*DepInfo, error) {
 		case DepTypeStamp:
 			hsh := m["Hash"]
 			if hsh == "" {
-				return nil, errors.New("invalid format of .dep")
+				return nil, errors.New("invalid format of .rec")
 			}
 			depInfo.stamp = hsh
 		default:
-			return nil, errors.New("invalid format of .dep")
+			return nil, errors.New("invalid format of .rec")
 		}
 	}
 	return &depInfo, nil
