@@ -345,6 +345,13 @@ func runScript(tgtOrig string, errs chan error, traced bool) error {
 	))
 
 	fdNum := 0
+	cmd.ExtraFiles = append(cmd.ExtraFiles, FdOODTgts)
+	cmd.Env = append(cmd.Env, fmt.Sprintf("%s=%d", EnvOODTgtsFd, 3+fdNum))
+	fdNum++
+	cmd.ExtraFiles = append(cmd.ExtraFiles, FdOODTgtsLock)
+	cmd.Env = append(cmd.Env, fmt.Sprintf("%s=%d", EnvOODTgtsLockFd, 3+fdNum))
+	fdNum++
+
 	if FdStatus == nil {
 		cmd.Env = append(cmd.Env, fmt.Sprintf("%s=NO", EnvStatusFd))
 	} else {
